#!/opt/concourse/worker/volumes/live/ce16c7e8-1b9d-4ed5-71a0-f56450a96636/volume/anaconda-navigator_1619554649950/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/Anaconda-Navigator.app/Contents/MacOS/python
from subprocess import call

call(["/opt/concourse/worker/volumes/live/ce16c7e8-1b9d-4ed5-71a0-f56450a96636/volume/anaconda-navigator_1619554649950/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/bin/anaconda-navigator"])
